#!/bin/sh

function usage () {
	cat <<EOF
Usage: $0 <bufilefullpath>
No arguments
EOF
	exit 2
}

[ $# -lt 1 ] && [ $# -gt 2 ] && usage

WD=$(echo $0 | awk '{sub(/\/[^\/]*?$/,""); print}')
source $WD/variables.sh
BUUSB=$1;

echo Connecting to the USB...
# 'mount /mnt/usb'   # FIXME short-circuited for now. 
# check if mounted, if not exit 10
echo Done!; echo;

# Unpack backup from USB to SDA card
echo 'Uploading...'
BUFILES=$(tar -jtf $BUUSB)
rm -rf $BUDIR/*
tar -C $BUDIR -jxf $BUUSB
for i in $BUFILES; \
	do \
	[ -e $BUDIR/$i ] || { echo 'Could not copy backup to SD card'; exit 2; }; \
	done;
echo Done!; echo;

echo 'Checking file integrity...'
cd $BUDIR
for i in $(ls | awk -F '.' '{print $1}' | uniq); \
	do \
	[ $(sha1sum $i.tgz | awk '/$i/ {print $1}') -eq $(cat $i.sha1 | awk '/$i/ {print $1}') ] || \
		{ echo 'File integrity check failed.'; exit 2; }; \
	done;
cd $WD
echo Done!; echo;

echo 'Unpacking Backup...'
cd $BUDIR
for i in `ls $BUDIR/*.tgz`; \
do \
	BUFILES=$(tar -jtf $i); \
	STEM=$(echo $i | awk '{sub(/^.*\//,""); sub(/\..*$/,""); print}'); \
	mkdir -p $STEM; \
	tar -C $BUDIR/$STEM/ -jxf $i; \
	for n in $BUFILES; \
	do \
		[ -e $BUDIR/$STEM/$n ] || { echo 'Could not unpack backup.'; }; \
	done; \
done;
cd $WD
echo Done!; echo;

