#!/bin/sh
###########################################################
#
# ------------------------------------------------------
# Copyright(c) 2008: Southwest Microwave, Inc.
# All rights reserved. Proprietary and confidential.
# Unauthorized reproduction, distribution , 
# modification is not permitted.
# ------------------------------------------------------
#
# FILENAME : 
#     /smi/config/comms/mkTs7400Eth0.sh
#
# PURPOSE  : 
#     update kernel network interface eth0 
#     restart network interface eth0
#     update apache2 configuration file
#     restart apache2 daemon
#
# HISTORY:
#     Sep 19, 2008 : JFD : update /etc/hosts with host name
#     Sep 5, 2008: JFD : Moved apache configuration 
#                        update to mkApache2Conf.sh
# 
# --------------------------------------------------------
# CVS Keywords : Automatically generated , do not edit
# --------------------------------------------------------
# $Source: /usr/local/cvsroot/cm_II/smi/config/comms/mkTs7400Eth0.sh,v $
# $Revision: 1.9 $
# $Date: 2008/09/19 17:09:04 $
# 
###########################################################
function usage () {
	cat <<EOF
Usage: $0 
Example:
	$0
EOF
	exit 1
}

[ $# -ne 0 ] && usage

###############################################################
# setup directory paths
###############################################################
WD="/smi/config/comms"
TD="/smi/etc"
FD="/etc"

################################################################
# networking files
################################################################
THEFILES="sysconfig/network_cfg sysconfig/ifcfg-eth0 hostname resolv.conf"
TMPFILES=""
BKPFILES=""
ETCFILES=""
APACFILE=""
ALLFILES=""
for i in $THEFILES
do
	TMPFILES="$TMPFILES $TD/$i"
	BKPFILES="$BKPFILES $TD/$i.bk"
	ETCFILES="$ETCFILES $FD/$i"
done

###############################################################
# test targets
###############################################################
ALLFILES="$TMPFILES $BKPFILES $ETCFILES $APACFILE"

for i in $ALLFILES
do
        chown www-data:www-data $i 
	touch $i || { echo "Could not write to network files"; exit 1; }
done

######################################################################
# update the temp networking files with data from site installer's
# configuration file /smi/config/comms/smiCmLan.data
######################################################################
awk -f $WD/mkNetCfg.awk $WD/smiCmLan.data     > $TD/sysconfig/network_cfg
awk -f $WD/mkIfCfgEth0.awk $WD/smiCmLan.data  > $TD/sysconfig/ifcfg-eth0
awk -f $WD/mkHostname.awk $WD/smiCmLan.data   > $TD/hostname
awk -f $WD/mkResolvConf.awk $WD/smiCmLan.data > $TD/resolv.conf
chmod -R 777 $TD/*
chown www-data:www-data $TD/*

#######################################################################
# perform backup of original /etc/sysconfig files
#######################################################################
for i in $THEFILES
do
	cp $FD/$i $TD/$i.bk || { echo "Could not write to backup network files"; exit 2; }
	chmod 777 $TD/*
	chown www-data:www-data $TD/*
done

######################################################################
# update networking files 
#       /etc/sysconfig/ifcfg-eth0, 
#       /etc/sysconfig/network_cfg
#       /etc/hostname
#       /etc/resolv.conf 
######################################################################
for i in $THEFILES
do
	chmod 777 $FD/$i
	chown www-data:www-data $FD/$i 
	cp $TD/$i $FD/$i || { echo "Could not write to final network files"; exit 3; }
done

######################################################################
# update /etc/hosts with new host name
######################################################################
sudo /smi/config/comms/mkHosts.sh

######################################################################
# update apache config with new ipaddress and port number 
######################################################################
sudo /smi/config/comms/mkHttpdConf.sh

######################################################################
# done
######################################################################
exit 0;

