<?php
require_once('reference.class.php');

class Devices {

// internal variables
var $device = array();
var $byNaddr;
var $version;

var $currDiscovery;
var $numCurr = 0;
var $newDiscovery;
var $numNew = 0;

var $default =
	Array
		(
			'deviceId'   =>  0,
			'cfgCsum'    => -1,
			'ftype'      => -1,
			'tagName'    => '""',
			'nAddr'      => -1,
			'macAddress' => '',
			'ttyPort'    => -1,
			'pollState'  => -1,
			'devStatus'  => -1,
			'pmaster'    => -1
		);

var $params =
	Array
		(
			'version'    => array('/^[0-9]+\.[0-9]+\.[0-9]+$/',NULL,NULL,FALSE),
			'deviceId'   => array('/^(-1|[0-9]{1,2})$/',-1,15,-1),
			'cfgCsum'    => array('/^(-1|0x[0-9a-fA-F]{1,4})$/',NULL,NULL,-1),
			'ftype'      => array('/^(-1|0x0?[1-6]00$)/',NULL,NULL,-1),
			'tagName'    => array('/^"?.{0,64}"?$/',NULL,NULL,''),
			'nAddr'      => array('/^-?[0-9]{1,3}$/',-1,239,-1),
			'macAddress' => array('/^(|([0-9a-fA-F]{0,2}:){5}[0-9a-fA-F]{0,2})$/',NULL,NULL,''),
			'ttyPort'    => array('/^-?[0-9]*$/',0,9,-1),
			'pollState'  => array('/^(0|1|-1)$/',0,1,-1),
			'devStatus'  => array('/^(-1|0x[0-9a-fA-F]{1,4})$/',NULL,NULL,FALSE),
			'pmaster'    => array('/^(0|1|-1)$/',0,1,-1)
		);

function Devices() {
	return;
}

function validParam($p,$v) {
	if(!isset($this->params[$p])) {
		return(FALSE);
	}
	if((strlen($this->params[$p][0]) > 0) && (preg_match($this->params[$p][0],$v) != 1)) {
		return(FALSE);
	}
	if(($this->params[$p][3] !== FALSE) && ($v == $this->params[$p][3])) {
		return(TRUE);
	}
	if(($this->params[$p][1] !== NULL) && ($this->params[$p][2] !== NULL) &&
		($v < $this->params[$p][1] || $v > $this->params[$p][2])
	) {
		return(FALSE);
	}
	return(TRUE);
}

function listDeviceFiles() {
	global $ref;

	$a = array();
	for($i=1; $i<=$ref->smiCmDevMax; $i++) {
		$a[] = $ref->deviceFileName($i);
	}
	return $a;
}

function getCurrentStatus($deviceNumber = null, $exist = false)
{
	global $ref;

	$returnExist = false;

	exec ( $ref->CGIDIR . '/Status', $output);
	//print_r($this->device);
	foreach($output as $deviceStatus)
	{
		if(empty($deviceStatus))
		{
			continue;
		}
		else
		{
			$thisDeviceStatus = explode(' ', $deviceStatus);
			if(isset($deviceNumber) && ($deviceNumber - 1) == $thisDeviceStatus[0])
			{
				$this->device[$deviceNumber]->devStatus = $thisDeviceStatus[1];
			}
			else
			{
				if($exist)
				{
					$returnExist = (empty($thisDeviceStatus[1]) ? false : true);
				}
				else
				{
					$this->device[($thisDeviceStatus[0] + 1)]->devStatus = $thisDeviceStatus[1];
				}
			}
		}
	}

	if($exist)
	{
		return $returnExist;
	}
}

function getCurrDiscoveryStatus()
{
	global $ref;

	exec ( $ref->CGIDIR . '/Status', $output);

	foreach($output as $deviceStatus)
	{
		if(empty($deviceStatus))
		{
			continue;
		}
		else
		{
			$thisDeviceStatus = explode(' ', $deviceStatus);
			if($thisDeviceStatus[1] != 0)
				$this->currDiscovery[$thisDeviceStatus[0]][devStatus] = $thisDeviceStatus[1];
		}
	}
}

function readDeviceFile($n,$path=NULL) {
	global $ref;
	global $message;

	$fn = $ref->deviceFileName($n,$path);
	$fh = fopen($fn,'r');
	if(!$fh) {
		$message->add('Device',"Could not open file $fn");
		return(FALSE);
	}

	$blank = FALSE;
	$fail = FALSE;
	while(!feof($fh)) {
		$l = trim(fgets($fh));
		if($l{0} == '#') {
			continue;
		}
		if(strlen($l) <= 1) {
			continue;
		}
		if(strpos($l,'version') === 0) {
            $version = trim(substr($l,1+strpos($l,'=')));
			$this->device[$n]->version = $version;
            continue;
        }
		if(strpos($l,'=') === FALSE) {
			$fail = "no = on line $l";
			break;
		}
		list($var,$val) = @explode('=',$l);
		$var = trim($var);
		$val = trim($val,"\" \t\n");
		if(!$this->validParam($var,$val)) {
			$fail = "no valid parameter ($var= $val)";
			break;
		}
		if(strcmp($var,'deviceId') === 0) {
			if($val != $n-1) {
				$fail = 'bad device ID';
				break;
			} else {
				$this->device[$n]->deviceId = $val;
			}
			continue;
		}
		$this->device[$n]->$var = $val;
		if(!$blank && isset($this->device[$n]->cfgCsum) && $this->device[$n]->cfgCsum==-1) {
			$blank = TRUE;
		}
	}
	fclose($fh);
	if($fail !== FALSE) {
//		unset($this->device[$n]);
		$message->add('Device',"Could not parse device file $fn, $fail");
		return(FALSE);
	}
	if($blank) {
//		unset($this->device[$n]);
	}
	foreach($this->device as $key => $d) {
		$this->byNaddr[$d->nAddr] =& $this->device[$key];
	}
	return(TRUE);
}

function readinDevices($path=NULL) {
	global $ref;
	global $message;
	for($i=1; $i<=$ref->smiCmDevMax; $i++) {
		if(!$this->readDeviceFile($i,$path)) {
			$message->add('Device',"Could not read device file $path/$i");
			return(FALSE);
		}
	}
	return(TRUE);
}

function byNaddr($naddr) {
	if(isset($this->byNaddr[$naddr])) {
		return($this->byNaddr[$naddr]);
	}
	return(FALSE);
}

function getDeviceByNaddr($naddr)
{
	foreach($this->device as $key => $device)
	{
		if($device->nAddr == $naddr)
		{
			return $key;
		}
	}

	return(FALSE);
}

function readDiscoveryFiles() {
	global $ref;
	$this->readDiscoveryFile($ref->currDevices,$this->currDiscovery,$this->numCurr);
	$this->readDiscoveryFile($ref->newDevices,$this->newDiscovery,$this->numNew);
}

function readDiscoveryFile($fn,&$table,&$numTable) {
	global $ref;
	global $message;

	$fh = fopen($fn,'r');
	if(!$fh) {
		$message->add('Device',"Could not open file $fn");
		return(FALSE);
	}
	$ver = FALSE;
	$h = FALSE;
	$numTable = 0;
	$headers = array();
	$table = array();
	while(!feof($fh)) {
		$l = trim(fgets($fh),"\n");
		if($l{0} == '#') { continue; }
		if(strlen($l) < 1) { continue; }
		if($ver) {
			$a = @explode(',',$l);
			if(!$a || count($a) <= 0) { continue; }
			if(!$h) {
				foreach($a as $k=>$v) {
					$headers[] = trim($v);
				}
				$h = TRUE;
			} else {
				foreach($a as $k=>$v) {
					$table[$numTable][$headers[$k]] = trim($v);
				}
				$table[$numTable]['ftypestring'] = $ref->deviceString($table[$numTable]['ftype']);
				$table[$numTable]['tagName'] = '"'.$table[$numTable]['tagName'].'"';
				$table[$numTable]['pollState'] = 1;
				if($table[$numTable]['ftypestring'] === '') {
					unset($table[$numTable]);
				} else {
					$numTable++;
				}
			}
		} elseif(strpos($l,'version') === 0) {
			$ver = TRUE;
		}
	}
	fclose($fh);
}



function newDisc2Device() {
	unset($this->device);
	$this->device = array();

	foreach($this->currDiscovery as $n => $curr) {
		$this->device[$n] = array();
		foreach($this->currDiscovery[$n] as $var => $val) {
			$this->device[$n][$var] = $val;
		}
		$this->device[$n]['deviceId'] = $n;
		unset($this->device[$n]['ftypestring']);
	}

	return(TRUE);
}


function writeDeviceFiles($path=NULL) {
	global $ref;
	if($path===NULL) {
		$path = $ref->smiCmDevPath;
	}
	for($n=1; $n<=$ref->smiCmDevMax; $n++) {
		$this->writeDeviceFile($n,$path);
	}
	return(TRUE);
}



function writeDeviceFile($n,$path=NULL) {
	global $ref;

	if($path===NULL) {
		$path = $ref->smiCmDevPath;
	}

	$i = $n-1;
	$fn = $ref->deviceFileNameDef($n,$path);
	$tfn = $ref->deviceFileNameTmp($n,$path);
	$ffn = $ref->deviceFileName($n,$path);
	$fh = fopen($fn,'r');
	if(!$fh) {
		$message->add('Device',"Could not open file $fn");
		return(FALSE);
	}
	$tfh = fopen($tfn,'w');
	if(!$tfh) {
		$message->add('Device',"Could not open file $tfn");
		return(FALSE);
	}
	$ndb = FALSE; // no duplicate blank lines
	while(!feof($fh)) {
		$l = trim(fgets($fh));
		if($l{0} == '#') {
			fwrite($tfh,"$l\n");
			$ndb = FALSE;
			continue;
		} elseif(strpos($l,'version') === 0) {
			list($var,$val) = @explode('=',$l);
			$val = trim($val,"\" \t\n");
			$this->device[$i]['version'] = $val;
			continue;
		} else {
			break;
		}
	}
	fclose($fh);
	foreach($this->params as $var => $a) {
		if(isset($this->device[$i][$var])) {
			if(strcmp($this->default[$var],'""') == 0) {
				fwrite($tfh,"$var= \"".trim($this->device[$i][$var],'" ')."\" \n");
			} else {
				fwrite($tfh,"$var= ".$this->device[$i][$var]." \n");
			}
		} else {
			if(strcmp($var,'deviceId') == 0) {
				fwrite($tfh,"$var= ".($n-1)." \n");
			} else {
				fwrite($tfh,"$var= ".$this->default[$var]." \n");
			}
		}
	}
	fclose($tfh);
	system("sed -i -e 's/smiCmDev_[0-9]*.def/smiCmDev_$n.data/' $tfn");
	system("mv $tfn $ffn");
	return(TRUE);
}



function changeTag($nAddr,$tag) {
	global $ref;
	global $message;
	if($nAddr == -1) {
		return(FALSE);
	}
	$status = 0;
	$findfile = 'grep \'nAddr= '.$nAddr.' \' '.$ref->DEVDIR.'/*.data | awk -F \':\' \'{print $1}\'';
	$fn = exec($findfile);
	if(!$fn) {
		$message->add('Device',"Could not find file: $findfile");
		return(FALSE);
	}
	$changetag = 'sed -i -e \'s/tagName= .* /tagName= "'.$tag.'" /\' '.$fn;
	exec($changetag,$out,$ret);
	if($ret != 0) {
		$message->add('Device',"Could not execute command $($changetag), ".implode('\n',$out));
		return(FALSE);
	}
	return(TRUE);
}



function changeCsum($nAddr,$csum) {
	global $ref;
	global $message;
	if($nAddr <= -1 || trim($nAddr) === '') {
		return(FALSE);
	}
	$status = 0;
	$findfile = 'grep \'nAddr= '.$nAddr.' \' '.$ref->DEVDIR.'/*.data | awk -F \':\' \'{print $1}\'';
	$fn = exec($findfile);
	if(!$fn) {
		$message->add('Device',"Could not find file: $findfile");
		return(FALSE);
	}
	$csum = sprintf("%x",$csum);
	$changecsum = 'sed -i -e \'s/cfgCsum= .* /cfgCsum= 0x'.$csum.' /\' '.$fn;
	exec($changecsum,$out,$ret);
	if($ret != 0) {
		$message->add('Device',"Could not execute command $($changecsum), ".implode('\n',$out));
		return(FALSE);
	}
	return(TRUE);
}



}

?>
