<?php
class Login {


// config variables
var $max_attempts = 3;
var $login_expiry = 600; // seconds that a session should last FIXME: change to 600
var $allowChange; // declared in allowChange()

/* Security Policies:
 *
 * smiAdmin, 'admin', Field installer and administrator functions.
 * - network and modem config
 * - Can change admin password only.
 * - Can restore the account file to defaults
 * - Device discovery
 * - Zone administration
 * - Can upload files
 *
 * smiSuper, 'super', Factory installer functions.
 * - network and modem config
 * - set smiSuper password, and reset accounts to defaults.
 * 
 * smiUser
 * - no write access at all, but can view.
 */
var $authTable = array(
'home'           => TRUE,
'login'          => TRUE,
'logout'         => TRUE,
'accounts'       => array('admin','super','user'),
'network'        => array('admin','super','user'),
'devicecontrol'  => array('admin','super'),
'zonecontrol'    => array('admin','super'),
'zonereport.txt' => TRUE,
'filetransfer'   => array('admin','super'),
'logs'           => array('admin','super','user')
);

// internal variables
var $attempts = 0;
var $loggedin = FALSE; 
var $username = '';
var $password = '';
var $logintype = 'none';
var $version = FALSE;
var $redirect = FALSE; // if TRUE, then the login class wants index.php to redirect to the main page.
var $userData = array();
var $userCount = 0;
var $cursor;		// Adding or changing accounts will be pointed to by this cursor
var $n2id = array();
var $lastlogin;

var $params = 
	Array
		(
			'type'    => array('/^(user|super|admin|none)$/',NULL,NULL,'none')
		);


function Login($off=FALSE) {
	// DO NOT place any code here.  You must de-serialize the object 
	// from the session variable, then run prepage().

	if($off) {
		return(TRUE);
	}

	//	This code is no-op, it only exists to get stuff into xgettext
/* TRANSLATOR: This word appears in the phrase "Can not delete users of type %s", and in a table of user information. */
	ADMIN;
/* TRANSLATOR: This word appears in the phrase "Can not delete users of type %s", and in a table of user information. */
	SUPER;
/* TRANSLATOR: This word appears in the phrase "Can not delete users of type %s", and in a table of user information. */
	USER;
/* TRANSLATOR: This appears before each message that appears from the login page.  e.g: 
Login: Logged in as <user>
Login: You are not authorized to see this page */
	LOGIN;
}

function prepage() {
// Design Note:  Write any extra code after the if($this->authenticate())
// statement.  This tests if the user is logged in, and a failure to login
// means nothing else should be executed.
	global $page;
	global $subpage;
	global $message;
	$time = time();

	$this->readSmiCmUsers();
	if( strcmp($page,'logout') == 0 ) {
		$this->logout();
		return;
	}
	if($this->attempts > 0 && !(strcmp($page,'login') == 0)) {
		// They gave up trying to log in
		$this->attempts = 0;
		return;
	}
	if($this->lockfileIsset()) {
		$this->logout(FALSE);
		$message->add( LOGIN, 
			/* TRANSLATOR: Message: A different user has logged in, so logins are not allowed. */ 
			SESSION_INUSE
		);
		return;
	}
	if(strcmp($page,'login') == 0 && isset($_POST['ok'])) {
		// user is attempting to log in
		$this->attempts++;
		$this->username = addslashes($_POST['userName']);
		$this->password = addslashes($_POST['userPass']);
		// Fallthru
	}
	if($this->authenticate()) {
		if($this->attempts > 0) {
			// Successful login attempt
			$this->attempts = 0;
			$this->lastlogin = $time;
			$this->lockfileSet();
			$this->redirect = TRUE;
			$this->logLogin();
			$message->add( LOGIN, 
				/* TRANSLATOR: Message: user has logged in */ LOGGED_IN
			);
		}
		if(($time - $this->lastlogin) > $this->login_expiry) {
			if(strcmp($subpage,'discovery') == 0) {
				$this->lastlogin = $time;
				$this->lockfileSet();
			} else {
				$this->logout();
				$message->add( LOGIN, 
					/* TRANSLATOR: Message: user has automatically been logged out */ LOGIN_EXPIRED
				);
				return(FALSE);
			}
		} else {
			$this->lastlogin = $time;
			$this->lockfileSet();
		}
		$this->loggedin = TRUE;
	} else {
		if($this->attempts >= $this->max_attempts) {
			// More than max_attempts failed logins.
			$this->attempts = 0;
			$this->redirect = TRUE;
			$this->logLoginFailure();
			$message->add(LOGIN, 
				/* TRANSLATOR: User has failed to log in too many times. */ ERROR_FAILED_LOGIN_ATTEMPT
			);
			return; 
		} elseif($this->loggedin) {
			// Password changed between page loads, yikes!
			$this->attempts = 0;
			$this->redirect = TRUE;
			$this->loggedin = FALSE;
			$this->logAuthFailure();
			$message->add(LOGIN, 
				PASSWORD_WAS_CHANGED
			);
			return; 
		} elseif($this->attempts > 0) {
			$message->add(LOGIN, ERROR_WRONG_PASSWORD.' ('.$this->attempts.') ');
			return; 
		}
	}
}

function logout($unlink=TRUE) {
	global $message;

	$this->username = '';
	$this->password = '';
	$this->loggedin = FALSE;
	$this->logLogout();
	session_unset();
	session_destroy();
	foreach($_SESSION as $k=>$v) {
		unset($_SESSION[$k]);
	}
	if($unlink) { $this->lockfileRemove(); }
	$message->clear();
}

// TRUE:  lock file is set, lock the user out
// FALSE: lock file is not set, allow the user in
function lockfileIsset() {
	global $ref;
	global $message;
	if(!file_exists($ref->smiLockfile)) {
		return(FALSE);
	}
	if(!is_readable($ref->smiLockfile)) {
		// Naughty naughty, no playing with lockfiles.
		$message->add(LOGIN,
		ERROR_SESSION_LOCK_NOT_READABLE
		);
		return(TRUE);
	}
	if(!is_writable($ref->smiLockfile)) {
		// Naughty naughty, no playing with lockfiles.
		$message->add(LOGIN,
		ERROR_SESSION_LOCK_NOT_WRITEABLE
		);
		return(TRUE);
	}
	if(!isset($_COOKIE['PHPSESSID'])) {
		$message->add(LOGIN,
		ERROR_NO_COOKIES
		);
		return(TRUE);
	}
	if(strcmp(file_get_contents($ref->smiLockfile),$_COOKIE['PHPSESSID']) === 0) {
		return(FALSE);
	}
	if((time() - filectime($ref->smiLockfile)) > $this->login_expiry) {
		return(FALSE);
	}
	return(TRUE);
}

function lockfileSet() {
	global $ref;
	$fh = fopen($ref->smiLockfile,'w+');
	if($fh === FALSE) {
		$message->add('Login: Could not set lockfile');
		return(FALSE);
	}
	if(fwrite($fh,$_COOKIE['PHPSESSID']) === FALSE) {
		$message->add('Login: Could not set lockfile');
		return(FALSE);
	}
	if(fclose($fh) === FALSE) {
		$message->add('Login: Could not set lockfile');
		return(FALSE);
	}
	return(TRUE);
}

function lockfileRemove() {
	global $ref;
	if(file_exists($ref->smiLockfile)) {
		unlink($ref->smiLockfile);
	}
}

function findEmptyUserSlot() {
	if($this->maxUsers()) {
		$this->cursor = FALSE;
		return(FALSE);
	}
	reset($this->userData);
	while(list($k,$u) = @each($this->userData)) {
		if(strlen($u['name']) <= 0) {
			$this->cursor =& $this->userData[$k];
			return($k);
		}
	}
	return(FALSE);
}

function findDuplicateName($name) {
	reset($this->userData);
	while(list($k,$u) = @each($this->userData)) {
		if(strcmp($u['name'],$name) === 0) {
			return(TRUE);
		}
	}
	return(FALSE);
}

function authenticate() {
	if(!isset($this->username) || !isset($this->n2id[$this->username])) {
		return(FALSE);
	}
	$uid = $this->n2id[$this->username];
	if(!isset($uid) || !isset($this->userData[$uid])) {
		$this->logintype = '';
		return(FALSE);
	}
	if(strcmp($this->password,$this->userData[$uid]['pass']) == 0) {
		$this->logintype = $this->userData[$uid]['type'];
		return(TRUE);	
	}
	$this->logintype = '';
	return(FALSE);
}

//	column1 = user account identifier 
//	column2 = user name  ( up to 32 alpha characters )
//	column3 = customer password ( up to 8 alpha-numeric characters )
//	column4 = account type ( none, user, admin, super )
function readSmiCmUsers($path=NULL) {

	global $ref;
	global $message;

	if($path===NULL) {
		$path=$ref->smiCmUsersPath;
	}
	$fn = $path.'/'.$ref->smiCmUsers;
	if(!is_readable($fn)) {
		$message->add(LOGIN, sprintf(ERROR_FILE_NOT_FOUND,$fn));
		return(FALSE);
	}
	$fh = fopen($fn,'r');
	if(!$fh) {
		$message->add(LOGIN, sprintf(ERROR_FILE_NOT_READABLE,$fn));
		return(FALSE);
	}
	$this->version = FALSE;
	$this->userData = array();
	$this->userCount = 0;
	$this->n2id = array();
	$fail = FALSE;
	while(!feof($fh)) {
		$l = fgets($fh);
		if($l{0} == '#') {
			continue;
		}
		if(strlen($l) <= 2) {
			continue;
		}
		if(!$this->version && strpos($l,'version') === 0) {
			$this->version = trim(substr($l,1+strpos($l,'=')));
			continue;
		}
		$a = explode(',',$l);
		if(count($a) < 4) {
			$fail = "wrong column count on $l";
			continue;
		}
		if(!(
			$this->addChangeUser(
				trim($a[0],"\" \t\n"),
				trim($a[1],"\" \t\n"),
				trim($a[2],"\" \t\n"),
				trim(substr($a[3],1+strpos($a[3],'=')),"\" \t\n"))
		)) {
			$fail = "could not parse user on $l";
		}
	}
	fclose($fh);
	if($fail !== FALSE) {
		$message->add('Login',"Could not parse user file, $fail on $fn");
		return(FALSE);
	}
	return(TRUE);
}

function writeSmiCmUsers() {
	global $ref;
	global $message;

	$fn = $ref->smiCmUsersPath.'/'.$ref->smiCmUsers;
	$tfn = $fn.'.tmp';

	if(!is_writable($fn)) {
		$message->add(LOGIN, sprintf(ERROR_FILE_NOT_WRITEABLE,$fn));
		return(FALSE);
	}
	$fh = fopen($fn,'r');
	if(!$fh) {
		$message->add(LOGIN, sprintf(ERROR_FILE_CANNOT_OPEN,$fn));
		return(FALSE);
	}
	$smiCmUsers = array();
	while(!feof($fh)) {
		$smiCmUsers[] = $l = fgets($fh);
	}
	fclose($fh);
	$fh = fopen($tfn,'w');
	if(!$fh) {
		$message->add(LOGIN, sprintf(ERROR_FILE_CANNOT_OPEN_WRITING,$tfn));
		return(FALSE);
	}
	reset($smiCmUsers);
	while(list($k,$l) = @each($smiCmUsers)) {
		if($l{0} == '#') {
			fwrite($fh,$l);
			continue;
		}
		// Once the comments end, we know what to write.
		fwrite($fh,'version= '.$this->version." \n");
		for($i=0; $i<$ref->smiCmUsersMax; $i++) {
			$wl = $this->writeSmiCmUserDataLine($i);
			fwrite($fh,$wl);
		}
		break;
	}
	fclose($fh);
	if(system("mv $tfn $fn") === FALSE) {
		$message->add(LOGIN, sprintf(
			/* TRANSLATOR: Warning message.  Each %s will have two different filenames. */
			ERROR_COULD_NOT_MOVE, $tfn, $fn)
		);
	}
	return(TRUE);
}



function validParam($p,$v) {
	if(!isset($this->params[$p])) {
		return(FALSE);
	}
	if((strlen($this->params[$p][0]) > 0) && (preg_match($this->params[$p][0],$v) != 1)) {
		return(FALSE);
	}
	if(($this->params[$p][3] !== FALSE) && ($v == $this->params[$p][3])) {
		return(TRUE);
	}
	if(($this->params[$p][1] !== NULL) && ($this->params[$p][2] !== NULL) && 
		($v < $this->params[$p][1] || $v > $this->params[$p][2])
	) {
		return(FALSE);
	}
	return(TRUE);
}



function addChangeUser($id,$name,$pass,$type) {
	global $message;
	if(!$this->validParam('type',$type)) { 
		$message->add('Login',"Invalid parameter $type, id=$id, name=$name");
		return(FALSE); 
	}

	$this->userData[$id] = array(
		'id'		=> $id,
		'name'		=> $name,
		'pass'		=> $pass,
		'type'		=> $type
	);
	if(strlen($name) > 0) {
		$this->n2id[$name] = $id;
		$this->userCount++;
	}
	return(TRUE);
}



function writeSmiCmUserDataLine($id) {
	return(	$this->userData[$id]['id'].', '.
		'"'.$this->userData[$id]['name'].'", '.
		'"'.$this->userData[$id]['pass'].'", '.
		'type= '.$this->userData[$id]['type']." \n"
	);
}



function authForThisPage($page) {
	global $message;

	if(!isset($this->authTable[$page])) {
		return(TRUE);
	}
	if($this->ima($this->authTable[$page])) {
		return(TRUE);
	}
	$message->add(LOGIN, ERROR_NOT_AUTHORIZED);
	return(FALSE);
}

function ima($u) {
	if(is_bool($u)) {
		return($u);
	} elseif(is_array($u)) {
		foreach($u as $usertype) {
			if(strcmp($usertype,$this->logintype) == 0) {
				return(TRUE);
			}
		}
	} else {
		if(str_cmp($u,$this->logintype) == 0) {
			return(TRUE);
		}
	}
	return(FALSE);
}

function maxUsers() {
	global $ref;
	return($this->userCount >= $ref->smiCmUsersMax ? TRUE : FALSE);
}

function redirect() {
	if($this->redirect) {
		$this->redirect = FALSE;
		return(TRUE);
	}
	return(FALSE);
}

function loggedin() {
	return($this->loggedin ? TRUE : FALSE);
}

/* logs a Login command */
function logLogin() {
	// FIXME Write the rest of me
}

/* logs a Logout command */
function logLogout() {
	// FIXME Write the rest of me
}

/* logs a login Failure */
function logLoginFailure() {
	// FIXME Write the rest of me
}

/* logs an authentication failure */
function logAuthFailure() {
	// FIXME Write the rest of me
}

function setCursor($id) {
	$this->cursor =& $this->userData[$id];
}

function itsMe($id) {
	return(strcmp($this->username,$this->userData[$id]['name']) == 0 ? TRUE : FALSE);
}

function allowChange($id,$type=FALSE) {
	if($type === FALSE) {
		if($this->itsMe($id)) {
			return(TRUE);
		}
		$type = $this->userData[$id]['type'];
	}

	$this->allowChange['admin']['admin'] = TRUE;
	$this->allowChange['admin']['super'] = TRUE;
	$this->allowChange['admin']['user']  = TRUE;
	$this->allowChange['super']['admin'] = FALSE;
	$this->allowChange['super']['super'] = TRUE;
	$this->allowChange['super']['user']  = TRUE;
	$this->allowChange['user']['admin']  = FALSE;
	$this->allowChange['user']['super']  = FALSE;
	$this->allowChange['user']['user']   = FALSE; // Users can only change themselves

	if(isset($this->allowChange[$this->logintype][$type])) {
		return($this->allowChange[$this->logintype][$type]);
	}
	return(FALSE);
}

function allowDelete($id,$type=FALSE) {
	if($type === FALSE) {
		if($this->itsMe($id)) {
			return(FALSE);
		}
		$type = $this->userData[$id]['type'];
	}

	$this->allowDelete['admin']['admin'] = FALSE;
	$this->allowDelete['admin']['super'] = FALSE;
	$this->allowDelete['admin']['user']  = TRUE;
	$this->allowDelete['super']['admin'] = FALSE;
	$this->allowDelete['super']['super'] = FALSE;
	$this->allowDelete['super']['user']  = TRUE;
	$this->allowDelete['user']['admin']  = FALSE;
	$this->allowDelete['user']['super']  = FALSE;
	$this->allowDelete['user']['user']   = FALSE;

	if(isset($this->allowDelete[$this->logintype][$type])) {
		return($this->allowDelete[$this->logintype][$type]);
	}
	return(FALSE);
}

function deleteUser($id) {
	global $message;

	if($this->itsMe($id)) {
		$message->add(LOGIN,
			sprintf(
			/* TRANSLATOR: Warning Message */
			ERROR_DELETE_SELF
			)
		);
		return(FALSE);
	}

	$name = $this->userData[$id]['name'];
	$type = $this->userData[$id]['type'];
	if(strcmp($type,'user') !== 0 || !$this->allowChange($id)) {
		$message->add(LOGIN,
			sprintf(
			/* TRANSLATOR: Warning Message */
			ERROR_DELETE_TYPE,_($type)
			)
		);
	} else {
		$this->addChangeUser($id,'','','none');
		if($this->writeSmiCmUsers()) {
		$message->add(LOGIN, sprintf(
			/* TRANSLATOR: Confirmation message.  The user named %s has been deleted. */
			USER_DELETED,$name)
		);
		} else {
		$message->add(LOGIN, sprintf(
			/* TRANSLATOR: Confirmation message.  The user named %s has NOT been deleted. */
			ERROR_DELETE,$name)
		);
		}
		$this->readSmiCmUsers();
	}
}

function validatePassword($pass,$pass2) {
	global $message;

	if(strlen($pass) <= 0) {
		$message->add(LOGIN, ERROR_PASSAWORD);
		return(FALSE);
	}
	if($pass !== FALSE && strcmp($pass,$pass2) !== 0) {
		$message->add(LOGIN, ERROR_PASSAWORD_MATCH);
		return(FALSE);
	} 	
	if(preg_match('/[^a-zA-Z0-9]/',$pass)) {
		$message->add(LOGIN, ERROR_PASSAWORD_ALPHA);
		return(FALSE);
	}
	return(TRUE);
}

function validateName($name) {
	global $message;

	if(strlen($name) <= 0) {
		$message->add(LOGIN, ERROR_NO_NAME);
		return(FALSE);
	}
	if(!preg_match('/^[a-zA-Z][a-zA-Z0-9]*$/',$name)) {
		$message->add(LOGIN, ERROR_NAME_ALPHA);
		return(FALSE);
	}
	return(TRUE);
}

function newPassword($id,$name,$pass,$pass2) {
	global $message;

	if(!$this->allowChange($id)) {
		$message->add(LOGIN, ERROR_CHANGE_USER_TYPE);	
		return(TRUE);	
	}
	if(!$this->validatePassword($pass,$pass2)) {
		return(FALSE);
	}
	$this->addChangeUser(
		$id,
		$name,
		$pass,
		$this->userData[$id]['type']
	);
	if($this->writeSmiCmUsers()) {
		$message->add(LOGIN, sprintf(
			/* TRANSLATOR: Confirmation Message. */
			PASSWORD_CHANGED_USER,$name)
		);
	} else {
		$message->add(LOGIN, sprintf(
			/* TRANSLATOR: Confirmation Message. */
			ERROR_CHANGE_USER,$name)
		);
	}
	$this->readSmiCmUsers();
	return(TRUE);
}

function addNewUser($id,$name,$pass,$pass2) {
	global $message;

	if(!$this->validateName($name)) {
		$message->add(LOGIN,sprintf(
			/* TRANSLATOR: Error message.  */
			ERROR_VALIDATE_NAME,$name));
		return(FALSE);
	}
	if(!$this->validatePassword($pass,$pass2)) {
		$message->add(LOGIN,sprintf(
			/* TRANSLATOR: Error message.  */
			ERROR_VALIDATE_PASSWORD,$name));
		return(FALSE);
	}
	if($this->findDuplicateName($name)) {
		$message->add(LOGIN, sprintf(
			/* TRANSLATOR: Confirmation message.  User named %s has not been added to the list. */
			ERROR_USER_EXISTS,$name));
		return(TRUE);
	}
	$this->addChangeUser(
		$id,
		$name,
		$pass,
		USER
	);
	if($this->writeSmiCmUsers()) {
		$message->add(LOGIN, sprintf(
			/* TRANSLATOR: Confirmation message.  User named %s has been added to the list. */
			USER_ADDED, $name)
		);
	} else {
		$message->add(LOGIN, sprintf(
			/* TRANSLATOR: Confirmation message.  User named %s has been added to the list. */
			ERROR_USER_NO_ADD, $name)
		);
	}
	$this->readSmiCmUsers();
	return(TRUE);
}

function restoreDefaults() {
	global $message;
	global $ref;

	if(strcmp($this->logintype,'admin') !== 0) {
		$message->add(LOGIN,
			/* TRANSLATOR: Warning message. */
			ERROR_USER_RESTORE_DEFAULS
		);
		$this->readSmiCmUsers();
		return(FALSE);
	}
	$status;
	$out = system($ref->smiCmdUserRestore,$status);
	$this->readSmiCmUsers();
	if($status < 0) {
		$message->add(LOGIN,
			/* TRANSLATOR: Warning message. */
			ERROR_CANNOT_RESTORE_FILE
		);
		return(FALSE);
	}
	$message->add(LOGIN,
		/* TRANSLATOR: Warning message. */
		RESTORED_ACCOUNTS
	);
	return(TRUE);
}

}
?>
